@@ -1,7 +1,7 @@
 """Dependency injector declarative container unit tests."""
 
 import collections
-import unittest2 as unittest
+import unittest
 
 from dependency_injector import (
     containers,
@@ -41,6 +41,29 @@ class DeclarativeContainerTests(unittest.TestCase):
                                                     p31=ContainerC.p31,
                                                     p32=ContainerC.p32))
 
+    def test_providers_attribute_with_redefinition(self):
+        p1 = providers.Provider()
+        p2 = providers.Provider()
+
+        class ContainerA2(ContainerA):
+            p11 = p1
+            p12 = p2
+
+        self.assertEqual(
+            ContainerA.providers,
+            {
+                'p11': ContainerA.p11,
+                'p12': ContainerA.p12,
+            },
+        )
+        self.assertEqual(
+            ContainerA2.providers,
+            {
+                'p11': p1,
+                'p12': p2,
+            },
+        )
+
     def test_cls_providers_attribute(self):
         self.assertEqual(ContainerA.cls_providers, dict(p11=ContainerA.p11,
                                                         p12=ContainerA.p12))
@@ -201,7 +224,7 @@ class DeclarativeContainerTests(unittest.TestCase):
                          (_OverridingContainer1.p11,
                           _OverridingContainer2.p11))
 
-    def test_reset_last_overridding(self):
+    def test_reset_last_overriding(self):
         class _Container(containers.DeclarativeContainer):
             p11 = providers.Provider()
 
@@ -221,7 +244,7 @@ class DeclarativeContainerTests(unittest.TestCase):
         self.assertEqual(_Container.p11.overridden,
                          (_OverridingContainer1.p11,))
 
-    def test_reset_last_overridding_when_not_overridden(self):
+    def test_reset_last_overriding_when_not_overridden(self):
         with self.assertRaises(errors.Error):
             ContainerA.reset_last_overriding()
 
@@ -296,6 +319,26 @@ class DeclarativeContainerTests(unittest.TestCase):
         self.assertEqual(_Container1.p13(), 11)
         self.assertEqual(_Container2.p13(), 22)
 
+    def test_copy_with_replacing_subcontainer_providers(self):
+        # See: https://github.com/ets-labs/python-dependency-injector/issues/374
+        class X(containers.DeclarativeContainer):
+            foo = providers.Dependency(instance_of=str)
+
+        def build_x():
+            return X(foo='1')
+
+        class A(containers.DeclarativeContainer):
+            x = providers.DependenciesContainer(**X.providers)
+            y = x.foo
+
+        @containers.copy(A)
+        class B1(A):
+            x = providers.Container(build_x)
+
+        b1 = B1()
+
+        self.assertEqual(b1.y(), '1')
+
     def test_containers_attribute(self):
         class Container(containers.DeclarativeContainer):
             class Container1(containers.DeclarativeContainer):
@@ -369,7 +412,7 @@ class DeclarativeContainerTests(unittest.TestCase):
         class Services(containers.DeclarativeContainer):
             a = providers.Dependency()
             c = providers.Factory(C, a=a)
-            b = providers.Factory(B, fa=a.delegate())
+            b = providers.Factory(B, fa=a.provider)
 
         a = providers.Factory(A)
         assert isinstance(Services(a=a).c().a, A)  # ok
@@ -388,3 +431,68 @@ class DeclarativeContainerTests(unittest.TestCase):
         self.assertIsInstance(container.p31, providers.Provider)
         self.assertIsInstance(container.p32, providers.Provider)
         self.assertIs(container.p11.last_overriding, provider)
+
+    def test_parent_set_in__new__(self):
+        class Container(containers.DeclarativeContainer):
+            dependency = providers.Dependency()
+            dependencies_container = providers.DependenciesContainer()
+            container = providers.Container(ContainerA)
+
+        self.assertIs(Container.dependency.parent, Container)
+        self.assertIs(Container.dependencies_container.parent, Container)
+        self.assertIs(Container.container.parent, Container)
+
+    def test_parent_set_in__setattr__(self):
+        class Container(containers.DeclarativeContainer):
+            pass
+
+        Container.dependency = providers.Dependency()
+        Container.dependencies_container = providers.DependenciesContainer()
+        Container.container = providers.Container(ContainerA)
+
+        self.assertIs(Container.dependency.parent, Container)
+        self.assertIs(Container.dependencies_container.parent, Container)
+        self.assertIs(Container.container.parent, Container)
+
+    def test_resolve_provider_name(self):
+        self.assertEqual(ContainerA.resolve_provider_name(ContainerA.p11), 'p11')
+
+    def test_resolve_provider_name_no_provider(self):
+        with self.assertRaises(errors.Error):
+            ContainerA.resolve_provider_name(providers.Provider())
+
+    def test_child_dependency_parent_name(self):
+        class Container(containers.DeclarativeContainer):
+            dependency = providers.Dependency()
+
+        with self.assertRaises(errors.Error) as context:
+            Container.dependency()
+        self.assertEqual(
+            str(context.exception),
+            'Dependency "Container.dependency" is not defined',
+        )
+
+    def test_child_dependencies_container_parent_name(self):
+        class Container(containers.DeclarativeContainer):
+            dependencies_container = providers.DependenciesContainer()
+
+        with self.assertRaises(errors.Error) as context:
+            Container.dependencies_container.dependency()
+        self.assertEqual(
+            str(context.exception),
+            'Dependency "Container.dependencies_container.dependency" is not defined',
+        )
+
+    def test_child_container_parent_name(self):
+        class ChildContainer(containers.DeclarativeContainer):
+            dependency = providers.Dependency()
+
+        class Container(containers.DeclarativeContainer):
+            child_container = providers.Container(ChildContainer)
+
+        with self.assertRaises(errors.Error) as context:
+            Container.child_container.dependency()
+        self.assertEqual(
+            str(context.exception),
+            'Dependency "Container.child_container.dependency" is not defined',
+        )
