@@ -190,6 +190,64 @@ class FactoryTests(AsyncTestCase):
 
         self.assertIsNot(service1.client, service2.client)
 
+    def test_injection_error(self):
+        async def init_resource():
+            raise Exception('Something went wrong')
+
+        class Container(containers.DeclarativeContainer):
+            resource_with_error = providers.Resource(init_resource)
+
+            client = providers.Factory(
+                Client,
+                resource1=resource_with_error,
+                resource2=None,
+            )
+
+        container = Container()
+
+        with self.assertRaises(Exception) as context:
+            self._run(container.client())
+        self.assertEqual(str(context.exception), 'Something went wrong')
+
+    def test_injection_runtime_error_async_provides(self):
+        async def create_client(*args,  **kwargs):
+            raise Exception('Something went wrong')
+
+        class Container(containers.DeclarativeContainer):
+            resource = providers.Resource(init_resource, providers.Object(RESOURCE1))
+
+            client = providers.Factory(
+                create_client,
+                resource1=resource,
+                resource2=None,
+            )
+
+        container = Container()
+
+        with self.assertRaises(Exception) as context:
+            self._run(container.client())
+        self.assertEqual(str(context.exception), 'Something went wrong')
+
+    def test_injection_call_error_async_provides(self):
+        async def create_client():  # <-- no args defined
+            ...
+
+        class Container(containers.DeclarativeContainer):
+            resource = providers.Resource(init_resource, providers.Object(RESOURCE1))
+
+            client = providers.Factory(
+                create_client,
+                resource1=resource,
+                resource2=None,
+            )
+
+        container = Container()
+
+        with self.assertRaises(TypeError) as context:
+            self._run(container.client())
+        self.assertIn("create_client() got", str(context.exception))
+        self.assertIn("unexpected keyword argument", str(context.exception))
+
     def test_attributes_injection(self):
         class ContainerWithAttributes(containers.DeclarativeContainer):
             resource1 = providers.Resource(init_resource, providers.Object(RESOURCE1))
@@ -236,6 +294,98 @@ class FactoryTests(AsyncTestCase):
 
         self.assertIsNot(service1.client, service2.client)
 
+    def test_attributes_injection_attribute_error(self):
+        class ClientWithException(Client):
+            @property
+            def attribute_set_error(self):
+                return None
+
+            @attribute_set_error.setter
+            def attribute_set_error(self, value):
+                raise Exception('Something went wrong')
+
+        class Container(containers.DeclarativeContainer):
+            resource = providers.Resource(init_resource, providers.Object(RESOURCE1))
+
+            client = providers.Factory(
+                ClientWithException,
+                resource1=resource,
+                resource2=resource,
+            )
+            client.add_attributes(attribute_set_error=123)
+
+        container = Container()
+
+        with self.assertRaises(Exception) as context:
+            self._run(container.client())
+        self.assertEqual(str(context.exception), 'Something went wrong')
+
+    def test_attributes_injection_runtime_error(self):
+        async def init_resource():
+            raise Exception('Something went wrong')
+
+        class Container(containers.DeclarativeContainer):
+            resource = providers.Resource(init_resource)
+
+            client = providers.Factory(
+                Client,
+                resource1=None,
+                resource2=None,
+            )
+            client.add_attributes(resource1=resource)
+            client.add_attributes(resource2=resource)
+
+        container = Container()
+
+        with self.assertRaises(Exception) as context:
+            self._run(container.client())
+        self.assertEqual(str(context.exception), 'Something went wrong')
+
+    def test_async_instance_and_sync_attributes_injection(self):
+        class ContainerWithAttributes(containers.DeclarativeContainer):
+            resource1 = providers.Resource(init_resource, providers.Object(RESOURCE1))
+
+            client = providers.Factory(
+                Client,
+                resource1,
+                resource2=None,
+            )
+            client.add_attributes(resource2=providers.Object(RESOURCE2))
+
+            service = providers.Factory(
+                Service,
+                client=None,
+            )
+            service.add_attributes(client=client)
+
+        container = ContainerWithAttributes()
+
+        client1 = self._run(container.client())
+        client2 = self._run(container.client())
+
+        self.assertIsInstance(client1, Client)
+        self.assertIs(client1.resource1, RESOURCE1)
+        self.assertIs(client1.resource2, RESOURCE2)
+
+        self.assertIsInstance(client2, Client)
+        self.assertIs(client2.resource1, RESOURCE1)
+        self.assertIs(client2.resource2, RESOURCE2)
+
+        service1 = self._run(container.service())
+        service2 = self._run(container.service())
+
+        self.assertIsInstance(service1, Service)
+        self.assertIsInstance(service1.client, Client)
+        self.assertIs(service1.client.resource1, RESOURCE1)
+        self.assertIs(service1.client.resource2, RESOURCE2)
+
+        self.assertIsInstance(service2, Service)
+        self.assertIsInstance(service2.client, Client)
+        self.assertIs(service2.client.resource1, RESOURCE1)
+        self.assertIs(service2.client.resource2, RESOURCE2)
+
+        self.assertIsNot(service1.client, service2.client)
+
 
 class FactoryAggregateTests(AsyncTestCase):
 
@@ -505,6 +655,32 @@ class ProvidedInstanceTests(AsyncTestCase):
         self.assertIs(instance2.resource, RESOURCE1)
         self.assertIs(instance1.resource, instance2.resource)
 
+    def test_provided_attribute_error(self):
+        async def raise_exception():
+            raise RuntimeError()
+
+        class TestContainer(containers.DeclarativeContainer):
+            client = providers.Factory(raise_exception)
+
+        container = TestContainer()
+
+        with self.assertRaises(RuntimeError):
+            self._run(container.client.provided.attr())
+
+    def test_provided_attribute_undefined_attribute(self):
+        class TestClient:
+            def __init__(self, resource):
+                self.resource = resource
+
+        class TestContainer(containers.DeclarativeContainer):
+            resource = providers.Resource(init_resource, providers.Object(RESOURCE1))
+            client = providers.Factory(TestClient, resource=resource)
+
+        container = TestContainer()
+
+        with self.assertRaises(AttributeError):
+            self._run(container.client.provided.attr())
+
     def test_provided_item(self):
         class TestClient:
             def __init__(self, resource):
@@ -535,6 +711,28 @@ class ProvidedInstanceTests(AsyncTestCase):
         self.assertIs(instance2.resource, RESOURCE1)
         self.assertIs(instance1.resource, instance2.resource)
 
+    def test_provided_item_error(self):
+        async def raise_exception():
+            raise RuntimeError()
+
+        class TestContainer(containers.DeclarativeContainer):
+            client = providers.Factory(raise_exception)
+
+        container = TestContainer()
+
+        with self.assertRaises(RuntimeError):
+            self._run(container.client.provided['item']())
+
+    def test_provided_item_undefined_item(self):
+        class TestContainer(containers.DeclarativeContainer):
+            resource = providers.Resource(init_resource, providers.Object(RESOURCE1))
+            client = providers.Factory(dict, resource=resource)
+
+        container = TestContainer()
+
+        with self.assertRaises(KeyError):
+            self._run(container.client.provided['item']())
+
     def test_provided_method_call(self):
         class TestClient:
             def __init__(self, resource):
@@ -565,9 +763,44 @@ class ProvidedInstanceTests(AsyncTestCase):
         self.assertIs(instance2.resource, RESOURCE1)
         self.assertIs(instance1.resource, instance2.resource)
 
+    def test_provided_method_call_parent_error(self):
+        async def raise_exception():
+            raise RuntimeError()
+
+        class TestContainer(containers.DeclarativeContainer):
+            client = providers.Factory(raise_exception)
+
+        container = TestContainer()
+
+        with self.assertRaises(RuntimeError):
+            self._run(container.client.provided.method.call()())
+
+    def test_provided_method_call_error(self):
+        class TestClient:
+            def method(self):
+                raise RuntimeError()
+
+        class TestContainer(containers.DeclarativeContainer):
+            client = providers.Factory(TestClient)
+
+        container = TestContainer()
+
+        with self.assertRaises(RuntimeError):
+            self._run(container.client.provided.method.call()())
+
 
 class DependencyTests(AsyncTestCase):
 
+    def test_provide_error(self):
+        async def get_async():
+            raise Exception
+
+        provider = providers.Dependency()
+        provider.override(providers.Callable(get_async))
+
+        with self.assertRaises(Exception):
+            self._run(provider())
+
     def test_isinstance(self):
         dependency = 1.0
 
@@ -635,6 +868,47 @@ class DependencyTests(AsyncTestCase):
         self.assertEqual(dependency4, dependency)
 
 
+class ListTests(AsyncTestCase):
+
+    def test_provide(self):
+        # See issue: https://github.com/ets-labs/python-dependency-injector/issues/450
+        async def create_resource(param: str):
+            return param
+
+        class Container(containers.DeclarativeContainer):
+
+            resources = providers.List(
+                providers.Resource(create_resource, 'foo'),
+                providers.Resource(create_resource, 'bar')
+            )
+
+        container = Container()
+        resources = self._run(container.resources())
+
+        self.assertEqual(resources[0], 'foo')
+        self.assertEqual(resources[1], 'bar')
+
+
+class DictTests(AsyncTestCase):
+
+    def test_provide(self):
+        async def create_resource(param: str):
+            return param
+
+        class Container(containers.DeclarativeContainer):
+
+            resources = providers.Dict(
+                foo=providers.Resource(create_resource, 'foo'),
+                bar=providers.Resource(create_resource, 'bar')
+            )
+
+        container = Container()
+        resources = self._run(container.resources())
+
+        self.assertEqual(resources['foo'], 'foo')
+        self.assertEqual(resources['bar'], 'bar')
+
+
 class OverrideTests(AsyncTestCase):
 
     def test_provider(self):
@@ -816,3 +1090,86 @@ class AsyncTypingStubTests(AsyncTestCase):
         self.assertIs(service2.client.resource2, RESOURCE2)
 
         self.assertIsNot(service1.client, service2.client)
+
+
+class AsyncProvidersWithAsyncDependenciesTests(AsyncTestCase):
+
+    def test_injections(self):
+        # See: https://github.com/ets-labs/python-dependency-injector/issues/368
+        async def async_db_provider():
+            return {'db': 'ok'}
+
+        async def async_service(db=None):
+            return {'service': 'ok', 'db': db}
+
+        class Container(containers.DeclarativeContainer):
+
+            db = providers.Factory(async_db_provider)
+            service = providers.Singleton(async_service, db=db)
+
+        container = Container()
+        service = self._run(container.service())
+
+        self.assertEqual(service, {'service': 'ok', 'db': {'db': 'ok'}})
+
+
+class AsyncProviderWithAwaitableObjectTests(AsyncTestCase):
+
+    def test(self):
+        class SomeResource:
+            def __await__(self):
+                raise RuntimeError('Should never happen')
+
+        async def init_resource():
+            pool = SomeResource()
+            yield pool
+
+        class Service:
+            def __init__(self, resource) -> None:
+                self.resource = resource
+
+        class Container(containers.DeclarativeContainer):
+            resource = providers.Resource(init_resource)
+            service = providers.Singleton(Service, resource=resource)
+
+        container = Container()
+
+        self._run(container.init_resources())
+        self.assertIsInstance(container.service(), asyncio.Future)
+        self.assertIsInstance(container.resource(), asyncio.Future)
+
+        resource = self._run(container.resource())
+        service = self._run(container.service())
+
+        self.assertIsInstance(resource, SomeResource)
+        self.assertIsInstance(service.resource, SomeResource)
+        self.assertIs(service.resource, resource)
+
+    def test_without_init_resources(self):
+        class SomeResource:
+            def __await__(self):
+                raise RuntimeError('Should never happen')
+
+        async def init_resource():
+            pool = SomeResource()
+            yield pool
+
+        class Service:
+            def __init__(self, resource) -> None:
+                self.resource = resource
+
+        class Container(containers.DeclarativeContainer):
+            resource = providers.Resource(init_resource)
+            service = providers.Singleton(Service, resource=resource)
+
+        container = Container()
+
+        self.assertIsInstance(container.service(), asyncio.Future)
+        self.assertIsInstance(container.resource(), asyncio.Future)
+
+        resource = self._run(container.resource())
+        service = self._run(container.service())
+
+        self.assertIsInstance(resource, SomeResource)
+        self.assertIsInstance(service.resource, SomeResource)
+        self.assertIs(service.resource, resource)
