@@ -2,7 +2,7 @@
 
 import sys
 
-import unittest2 as unittest
+import unittest
 
 from dependency_injector import (
     providers,
@@ -22,6 +22,20 @@ class CallableTests(unittest.TestCase):
     def test_init_with_not_callable(self):
         self.assertRaises(errors.Error, providers.Callable, 123)
 
+    def test_init_optional_provides(self):
+        provider = providers.Callable()
+        provider.set_provides(object)
+        self.assertIs(provider.provides, object)
+        self.assertIsInstance(provider(), object)
+
+    def test_set_provides_returns_self(self):
+        provider = providers.Callable()
+        self.assertIs(provider.set_provides(object), provider)
+
+    def test_provided_instance_provider(self):
+        provider = providers.Callable(_example)
+        self.assertIsInstance(provider.provided, providers.ProvidedInstance)
+
     def test_call(self):
         provider = providers.Callable(lambda: True)
         self.assertTrue(provider())
@@ -65,7 +79,7 @@ class CallableTests(unittest.TestCase):
         provider = providers.Callable(_example) \
             .add_args(1, 2) \
             .set_args(3, 4)
-        self.assertEqual(provider.args, tuple([3, 4]))
+        self.assertEqual(provider.args, (3, 4))
 
     def test_set_kwargs(self):
         provider = providers.Callable(_example) \
