@@ -1,6 +1,6 @@
 """Dependency injector provided instance provider unit tests."""
 
-import unittest2 as unittest
+import unittest
 
 from dependency_injector import containers, providers
 
@@ -10,12 +10,20 @@ class Service:
         self.value = value
         self.values = [self.value]
 
-    def get_value(self):
+    def __call__(self):
         return self.value
 
     def __getitem__(self, item):
         return self.values[item]
 
+    def get_value(self):
+        return self.value
+
+    def get_closure(self):
+        def closure():
+            return self.value
+        return closure
+
 
 class Client:
     def __init__(self, value):
@@ -45,6 +53,15 @@ class Container(containers.DeclarativeContainer):
         Client,
         value=service.provided.get_value.call(),
     )
+    client_method_closure_call = providers.Factory(
+        Client,
+        value=service.provided.get_closure.call().call(),
+    )
+
+    client_provided_call = providers.Factory(
+        Client,
+        value=service.provided.call(),
+    )
 
 
 class ProvidedInstanceTests(unittest.TestCase):
@@ -71,6 +88,14 @@ class ProvidedInstanceTests(unittest.TestCase):
         client = self.container.client_method_call()
         self.assertEqual(client.value, 'foo')
 
+    def test_method_closure_call(self):
+        client = self.container.client_method_closure_call()
+        self.assertEqual(client.value, 'foo')
+
+    def test_provided_call(self):
+        client = self.container.client_provided_call()
+        self.assertEqual(client.value, 'foo')
+
     def test_call_overridden(self):
         value = 'bar'
         with self.container.service.override(Service(value)):
@@ -101,6 +126,44 @@ class ProvidedInstanceTests(unittest.TestCase):
         )
 
 
+class LazyInitTests(unittest.TestCase):
+
+    def test_provided_instance(self):
+        provides = providers.Object(object())
+        provider = providers.ProvidedInstance()
+        provider.set_provides(provides)
+        self.assertIs(provider.provides, provides)
+        self.assertIs(provider.set_provides(providers.Provider()), provider)
+
+    def test_attribute_getter(self):
+        provides = providers.Object(object())
+        provider = providers.AttributeGetter()
+        provider.set_provides(provides)
+        provider.set_name('__dict__')
+        self.assertIs(provider.provides, provides)
+        self.assertEqual(provider.name, '__dict__')
+        self.assertIs(provider.set_provides(providers.Provider()), provider)
+        self.assertIs(provider.set_name('__dict__'), provider)
+
+    def test_item_getter(self):
+        provides = providers.Object({'foo': 'bar'})
+        provider = providers.ItemGetter()
+        provider.set_provides(provides)
+        provider.set_name('foo')
+        self.assertIs(provider.provides, provides)
+        self.assertEqual(provider.name, 'foo')
+        self.assertIs(provider.set_provides(providers.Provider()), provider)
+        self.assertIs(provider.set_name('foo'), provider)
+
+    def test_method_caller(self):
+        provides = providers.Object(lambda: 42)
+        provider = providers.MethodCaller()
+        provider.set_provides(provides)
+        self.assertIs(provider.provides, provides)
+        self.assertEqual(provider(), 42)
+        self.assertIs(provider.set_provides(providers.Provider()), provider)
+
+
 class ProvidedInstancePuzzleTests(unittest.TestCase):
 
     def test_puzzled(self):
