@@ -2,7 +2,7 @@
 
 import asyncio
 
-import unittest2 as unittest
+import unittest
 
 from dependency_injector import containers, providers, resources, errors
 
@@ -29,6 +29,16 @@ class ResourceTests(unittest.TestCase):
     def test_is_provider(self):
         self.assertTrue(providers.is_provider(providers.Resource(init_fn)))
 
+    def test_init_optional_provides(self):
+        provider = providers.Resource()
+        provider.set_provides(init_fn)
+        self.assertIs(provider.provides, init_fn)
+        self.assertEqual(provider(), (tuple(), dict()))
+
+    def test_set_provides_returns_self(self):
+        provider = providers.Resource()
+        self.assertIs(provider.set_provides(init_fn), provider)
+
     def test_provided_instance_provider(self):
         provider = providers.Resource(init_fn)
         self.assertIsInstance(provider.provided, providers.ProvidedInstance)
@@ -203,7 +213,7 @@ class ResourceTests(unittest.TestCase):
         provider = providers.Resource(init_fn) \
             .add_args(1, 2) \
             .set_args(3, 4)
-        self.assertEqual(provider.args, tuple([3, 4]))
+        self.assertEqual(provider.args, (3, 4))
 
     def test_clear_args(self):
         provider = providers.Resource(init_fn) \
@@ -337,9 +347,9 @@ class ResourceTests(unittest.TestCase):
 
         self.assertEqual(
             repr(provider),
-            'Resource({0}, initialized={1})'.format(
-                init_fn,
-                provider.initialized,
+            '<dependency_injector.providers.Resource({0}) at {1}>'.format(
+                repr(init_fn),
+                hex(id(provider)),
             )
         )
 
@@ -449,14 +459,45 @@ class AsyncResourceTest(AsyncTestCase):
         self.assertTrue(provider.initialized)
         self.assertTrue(provider.is_async_mode_enabled())
 
-        # Disable default exception handling to prevent output
-        asyncio.get_event_loop().set_exception_handler(lambda loop, context: ...)
+        with self.assertRaises(RuntimeError):
+            self._run(future)
+
+        self.assertFalse(provider.initialized)
+        self.assertTrue(provider.is_async_mode_enabled())
+
+    def test_init_async_gen_with_error(self):
+        async def _init():
+            raise RuntimeError()
+            yield
+
+        provider = providers.Resource(_init)
+
+        future = provider()
+        self.assertTrue(provider.initialized)
+        self.assertTrue(provider.is_async_mode_enabled())
 
         with self.assertRaises(RuntimeError):
             self._run(future)
 
-        # Restore default exception handling
-        asyncio.get_event_loop().set_exception_handler(None)
+        self.assertFalse(provider.initialized)
+        self.assertTrue(provider.is_async_mode_enabled())
+
+    def test_init_async_subclass_with_error(self):
+        class _Resource(resources.AsyncResource):
+            async def init(self):
+                raise RuntimeError()
+
+            async def shutdown(self, resource):
+                pass
+
+        provider = providers.Resource(_Resource)
+
+        future = provider()
+        self.assertTrue(provider.initialized)
+        self.assertTrue(provider.is_async_mode_enabled())
+
+        with self.assertRaises(RuntimeError):
+            self._run(future)
 
         self.assertFalse(provider.initialized)
         self.assertTrue(provider.is_async_mode_enabled())
