@@ -4,9 +4,9 @@ import functools
 import itertools
 import sys
 
-import unittest2 as unittest
+import unittest
 
-from dependency_injector import providers
+from dependency_injector import providers, errors
 
 
 class SelectorTests(unittest.TestCase):
@@ -16,6 +16,32 @@ class SelectorTests(unittest.TestCase):
     def test_is_provider(self):
         self.assertTrue(providers.is_provider(providers.Selector(self.selector)))
 
+    def test_init_optional(self):
+        one = providers.Object(1)
+        two = providers.Object(2)
+
+        provider = providers.Selector()
+        provider.set_selector(self.selector)
+        provider.set_providers(one=one, two=two)
+
+        self.assertEqual(provider.providers, {'one': one, 'two': two})
+        with self.selector.override('one'):
+            self.assertEqual(provider(), one())
+        with self.selector.override('two'):
+            self.assertEqual(provider(), two())
+
+    def test_set_selector_returns_self(self):
+        provider = providers.Selector()
+        self.assertIs(provider.set_selector(self.selector), provider)
+
+    def test_set_providers_returns_self(self):
+        provider = providers.Selector()
+        self.assertIs(provider.set_providers(one=providers.Provider()), provider)
+
+    def test_provided_instance_provider(self):
+        provider = providers.Selector(self.selector)
+        self.assertIsInstance(provider.provided, providers.ProvidedInstance)
+
     def test_call(self):
         provider = providers.Selector(
             self.selector,
@@ -29,6 +55,28 @@ class SelectorTests(unittest.TestCase):
         with self.selector.override('two'):
             self.assertEqual(provider(), 2)
 
+    def test_call_undefined_provider(self):
+        provider = providers.Selector(
+            self.selector,
+            one=providers.Object(1),
+            two=providers.Object(2),
+        )
+
+        with self.selector.override('three'):
+            with self.assertRaises(errors.Error):
+                provider()
+
+    def test_call_selector_is_none(self):
+        provider = providers.Selector(
+            self.selector,
+            one=providers.Object(1),
+            two=providers.Object(2),
+        )
+
+        with self.selector.override(None):
+            with self.assertRaises(errors.Error):
+                provider()
+
     def test_call_any_callable(self):
         provider = providers.Selector(
             functools.partial(next, itertools.cycle(['one', 'two'])),
@@ -66,6 +114,19 @@ class SelectorTests(unittest.TestCase):
         self.assertIs(provider.one, provider_one)
         self.assertIs(provider.two, provider_two)
 
+    def test_getattr_attribute_error(self):
+        provider_one = providers.Object(1)
+        provider_two = providers.Object(2)
+
+        provider = providers.Selector(
+            self.selector,
+            one=provider_one,
+            two=provider_two,
+        )
+
+        with self.assertRaises(AttributeError):
+            _ = provider.provider_three
+
     def test_call_overridden(self):
         provider = providers.Selector(self.selector, sample=providers.Object(1))
         overriding_provider1 = providers.Selector(self.selector, sample=providers.Object(2))
@@ -77,6 +138,18 @@ class SelectorTests(unittest.TestCase):
         with self.selector.override('sample'):
             self.assertEqual(provider(), 3)
 
+    def test_providers_attribute(self):
+        provider_one = providers.Object(1)
+        provider_two = providers.Object(2)
+
+        provider = providers.Selector(
+            self.selector,
+            one=provider_one,
+            two=provider_two,
+        )
+
+        self.assertEqual(provider.providers, {'one': provider_one, 'two': provider_two})
+
     def test_deepcopy(self):
         provider = providers.Selector(self.selector)
 
