@@ -1,6 +1,6 @@
 """Dependency injector common unit tests."""
 
-import unittest2 as unittest
+import unittest
 
 from dependency_injector import __version__
 
