@@ -1,7 +1,16 @@
+import contextlib
 from decimal import Decimal
+import importlib
 import unittest
 
-from dependency_injector.wiring import wire, Provide, Closing
+from dependency_injector.wiring import (
+    wire,
+    Provide,
+    Provider,
+    Closing,
+    register_loader_containers,
+    unregister_loader_containers,
+)
 from dependency_injector import errors
 
 # Runtime import to avoid syntax errors in samples on Python < 3.5
@@ -56,6 +65,20 @@ class WiringTest(unittest.TestCase):
         service = test_function()
         self.assertIsInstance(service, Service)
 
+    def test_module_attributes_wiring(self):
+        self.assertIsInstance(module.service, Service)
+        self.assertIsInstance(module.service_provider(), Service)
+        self.assertIsInstance(module.undefined, Provide)
+
+    def test_module_attribute_wiring_with_invalid_marker(self):
+        from wiringsamples import module_invalid_attr_injection
+        with self.assertRaises(Exception) as context:
+            self.container.wire(modules=[module_invalid_attr_injection])
+        self.assertEqual(
+            str(context.exception),
+            'Unknown type of marker {0}'.format(module_invalid_attr_injection.service),
+        )
+
     def test_class_wiring(self):
         test_class_object = module.TestClass()
         self.assertIsInstance(test_class_object.service, Service)
@@ -89,6 +112,11 @@ class WiringTest(unittest.TestCase):
         service = instance.static_method()
         self.assertIsInstance(service, Service)
 
+    def test_class_attribute_wiring(self):
+        self.assertIsInstance(module.TestClass.service, Service)
+        self.assertIsInstance(module.TestClass.service_provider(), Service)
+        self.assertIsInstance(module.TestClass.undefined, Provide)
+
     def test_function_wiring(self):
         service = module.test_function()
         self.assertIsInstance(service, Service)
@@ -112,19 +140,23 @@ class WiringTest(unittest.TestCase):
     def test_configuration_option(self):
         (
             value_int,
+            value_float,
             value_str,
             value_decimal,
             value_required,
             value_required_int,
+            value_required_float,
             value_required_str,
             value_required_decimal,
         ) = module.test_config_value()
 
         self.assertEqual(value_int, 10)
+        self.assertEqual(value_float, 10.0)
         self.assertEqual(value_str, '10')
         self.assertEqual(value_decimal, Decimal(10))
         self.assertEqual(value_required, 10)
         self.assertEqual(value_required_int, 10)
+        self.assertEqual(value_required_float, 10.0)
         self.assertEqual(value_required_str, '10')
         self.assertEqual(value_required_decimal, Decimal(10))
 
@@ -161,6 +193,9 @@ class WiringTest(unittest.TestCase):
         }
         self.container.config.from_dict(config)
 
+        value_default = module.test_config_invariant()
+        self.assertEqual(value_default, 1)
+
         with self.container.config.switch.override('a'):
             value_a = module.test_config_invariant()
         self.assertEqual(value_a, 1)
@@ -200,6 +235,18 @@ class WiringTest(unittest.TestCase):
         self.container.unwire()
         self.assertIsInstance(submodule.test_function(), Provide)
 
+    def test_unwire_module_attributes(self):
+        self.container.unwire()
+        self.assertIsInstance(module.service, Provide)
+        self.assertIsInstance(module.service_provider, Provider)
+        self.assertIsInstance(module.undefined, Provide)
+
+    def test_unwire_class_attributes(self):
+        self.container.unwire()
+        self.assertIsInstance(module.TestClass.service, Provide)
+        self.assertIsInstance(module.TestClass.service_provider, Provider)
+        self.assertIsInstance(module.TestClass.undefined, Provide)
+
     def test_wire_multiple_containers(self):
         sub_container = SubContainer()
         sub_container.wire(
@@ -367,3 +414,30 @@ class WiringAsyncInjectionsTest(AsyncTestCase):
         self.assertIs(resource2, asyncinjections.resource2)
         self.assertEqual(asyncinjections.resource2.init_counter, 2)
         self.assertEqual(asyncinjections.resource2.shutdown_counter, 2)
+
+
+class AutoLoaderTest(unittest.TestCase):
+
+    container: Container
+
+    def setUp(self) -> None:
+        self.container = Container(config={'a': {'b': {'c': 10}}})
+        importlib.reload(module)
+
+    def tearDown(self) -> None:
+        with contextlib.suppress(ValueError):
+            unregister_loader_containers(self.container)
+
+        self.container.unwire()
+
+    @classmethod
+    def tearDownClass(cls) -> None:
+        importlib.reload(module)
+
+    def test_register_container(self):
+        register_loader_containers(self.container)
+        importlib.reload(module)
+        importlib.import_module('wiringsamples.imports')
+
+        service = module.test_function()
+        self.assertIsInstance(service, Service)
